@@ -42,6 +42,7 @@
 #include <stdlib.h>
 #include <string.h>
 #include <unistd.h>
+#include <ctype.h>
 #include "teo.h"
 
 void TeoSetErrorMessage(int code,...);
@@ -998,6 +999,39 @@ int TeoWriteHeader(TEOFILE *teofp){
   return teofp->fsize;
 }
 
+// read a non-negative integer from the source.
+// return -1 upon error
+
+static int read_number(FILE *fp) {
+  int number;
+  unsigned char ch;
+  
+  // Initialize return value
+  number = 0;
+  // Skip leading whitespace
+  do {
+    if ( ! fread(&ch, 1, 1, fp) ) {
+      return -1;
+    }
+    // Eat comments as whitespace
+    if ( ch == '#' ) {  // Comment is '#' to end of line
+      do {
+	if ( ! fread(&ch, 1, 1, fp) ) {
+	  return -1;
+	}
+      } while ( (ch != '\r') && (ch != '\n') );
+    }
+  } while ( isspace(ch) );
+  // Add up the number
+  do {
+    number *= 10;
+    number += ch-'0';
+    if ( ! fread(&ch, 1, 1, fp) ) {
+      return -1;
+    }
+  } while ( isdigit(ch) );
+  return(number);
+}
 
 int PnmReadHeader(TEOFILE *teofp){
   char line[TEO_MAXLINE];
@@ -1021,18 +1055,14 @@ int PnmReadHeader(TEOFILE *teofp){
   /* check magic */
   if(memcmp(magic,"P4",2) == 0){
     /* if PBM(raw) format */
-    do{
-      if(fgets(line,TEO_MAXLINE,teofp->fp)==NULL){
-	TeoSetErrorMessage(TEO_ER_READ_CANT_GET_SIZE,teofp->filename);
-	return  0;
-      }
-    } while((line)[0] == '#');
-
-    if(sscanf(line,"%d%d",&teofp->width,&teofp->height) != 2){
+    if( (teofp->width = read_number(teofp->fp)) < 0) {
       TeoSetErrorMessage(TEO_ER_READ_CANT_GET_SIZE,teofp->filename);
-      return 0;
+      return  0;
+    }
+    if( (teofp->height = read_number(teofp->fp)) < 0) {
+      TeoSetErrorMessage(TEO_ER_READ_CANT_GET_SIZE,teofp->filename);
+      return  0;
     }
-
     teofp->type = TEO_UNSIGNED;
     teofp->bit = 1;
     teofp->plane = 1;
@@ -1041,35 +1071,22 @@ int PnmReadHeader(TEOFILE *teofp){
     int graylevel;
 
     /* if PGM(raw) */
-    do{
-      if(fgets(line,TEO_MAXLINE,teofp->fp)==NULL){
-	TeoSetErrorMessage(TEO_ER_READ_CANT_GET_SIZE,teofp->filename);
-	return 0;
-      }
-    } while((line)[0] == '#');
-
-    if(sscanf(line,"%d%d",&teofp->width,&teofp->height) != 2){
+    if( (teofp->width = read_number(teofp->fp)) < 0) {
       TeoSetErrorMessage(TEO_ER_READ_CANT_GET_SIZE,teofp->filename);
-      return 0;
+      return  0;
     }
-
-    do{
-      if(fgets(line,TEO_MAXLINE,teofp->fp)==NULL){
-	TeoSetErrorMessage(TEO_ER_READ_CANT_GET_SIZE,teofp->filename);
-	return 0;
-      }
-    } while((line)[0] == '#');
-
-    if(sscanf(line,"%d",&graylevel) != 1){
-      TeoSetErrorMessage(TEO_ER_READ_CANT_GET_PTYPE,teofp->filename);
-      return 0;
+    if( (teofp->height = read_number(teofp->fp)) < 0) {
+      TeoSetErrorMessage(TEO_ER_READ_CANT_GET_SIZE,teofp->filename);
+      return  0;
+    }
+    if( (graylevel  = read_number(teofp->fp)) < 0) {
+      TeoSetErrorMessage(TEO_ER_READ_CANT_GET_SIZE,teofp->filename);
+      return  0;
     }
-
     if(graylevel > 255){
       TeoSetErrorMessage(TEO_ER_READ_WRONG_PSIZE,teofp->filename);
       return 0;
     }
-
     teofp->type = TEO_UNSIGNED;
     teofp->bit = 8;
     teofp->plane = 1;
@@ -1078,28 +1095,17 @@ int PnmReadHeader(TEOFILE *teofp){
     int graylevel;
 
     /* if PPM(raw) format */
-    do{
-      if(fgets(line,TEO_MAXLINE,teofp->fp)==NULL){
-	TeoSetErrorMessage(TEO_ER_READ_CANT_GET_SIZE,teofp->filename);
-	return 0;
-      }
-    } while((line)[0] == '#');
-
-    if(sscanf(line,"%d%d",&teofp->width,&teofp->height) != 2){
+    if( (teofp->width = read_number(teofp->fp)) < 0) {
       TeoSetErrorMessage(TEO_ER_READ_CANT_GET_SIZE,teofp->filename);
-      return 0;
+      return  0;
     }
-
-    do{
-      if(fgets(line,TEO_MAXLINE,teofp->fp)==NULL){
-	TeoSetErrorMessage(TEO_ER_READ_CANT_GET_PTYPE,teofp->filename);
-	return 0;
-      }
-    } while((line)[0] == '#');
-
-    if(sscanf(line,"%d",&graylevel) != 1){
-      TeoSetErrorMessage(TEO_ER_READ_CANT_GET_PTYPE,teofp->filename);
-      return 0;
+    if( (teofp->height = read_number(teofp->fp)) < 0) {
+      TeoSetErrorMessage(TEO_ER_READ_CANT_GET_SIZE,teofp->filename);
+      return  0;
+    }
+    if( (graylevel  = read_number(teofp->fp)) < 0) {
+      TeoSetErrorMessage(TEO_ER_READ_CANT_GET_SIZE,teofp->filename);
+      return  0;
     }
     if(graylevel > 255){
       TeoSetErrorMessage(TEO_ER_READ_WRONG_PSIZE,teofp->filename);
